
//******************************************************************************
//******************************* INITIALIZATION *******************************
//******************************************************************************

#include <LiquidCrystal.h>
#include <OneWire.h>
#include <elapsedMillis.h>

#define length 20.0     //Definition of LCD length for the bar graph power meter

LiquidCrystal lcd(7, 8, 9, 10, 11, 12);

OneWire  ds(2);        //Temperature sensor DS18B20 to Arduino pin #2

//Variables'declaration

float percent=100.0;  //Bar graph power meter
float a = 0.0;  //Bar graph power meter
unsigned char b;   //Bar graph power meter
int i = 0;  //Bar graph power meter
unsigned int peace;     //Bar graph power meter
unsigned int offset=0;   //Bar graph power meter "zero", between 0 et 1023
unsigned int val = 0;  //Bar graph power meter

byte j;  //Temperature measurement
byte present = 0;  //Temperature measurement
byte data[12];  //Temperature measurement
byte addr[8];  //Temperature measurement
  
int HighByte, LowByte, SignBit, Whole, Fract, TReading, Tc_100, FWhole;  //Temperature measurement

int Analog_Input_Port = 5;  //Power measurement
float Voltage = 0.0;  //Power measurement
float Power_W = 0.0;  //Power measurement
float Value = 0.0;  //Power measurement

const int Pin_Backlight = 13;  //LCD Backlight
unsigned long BacklightTime = 0;  //LCD Backlight

elapsedMillis timer = 0;   //LCD time management
unsigned int interval = 800;    //LCD time management

const int Pin_PTT = 3;  //PTT sensing
int PTTState = 0;  //PTT sensing

const int Pin_Mode_Button = 6;  //LCD mode button
int buttonState = 0;  //LCD mode button
int lastButtonState = 0;  //LCD mode button
int buttonPushCounter = 1;  //LCD mode button

//Definition of the LCD special characters

byte Empty[8] = {
  B00000,
  B00000,
  B00000,
  B00100,
  B00100,
  B00000,
  B00000,
  B00000};

byte Col1[8] = {
  B10000,
  B10000,
  B10000,
  B10100,
  B10100,
  B10000,
  B10000,
  B10000};

byte Col2[8] = {
  B11000,
  B11000,
  B11000,
  B11100,
  B11100,
  B11000,
  B11000,
  B11000};

byte Col3[8] = {
  B11100,
  B11100,
  B11100,
  B11100,
  B11100,
  B11100,
  B11100,
  B11100};

byte Col4[8] = {
  B11110,
  B11110,
  B11110,
  B11110,
  B11110,
  B11110,
  B11110,
  B11110};

byte Col5[8] = {
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111};
  
byte deg[8] = {
  B01110,
  B01010,
  B01110,
  B00000,
  B00000,
  B00000,
  B00000,
  B00000};  
 
//******************************************************************************
//******************************* SETUP ****************************************
//******************************************************************************

void setup() {
  
lcd.begin (20,2);  
lcd.clear();
  
pinMode(Pin_PTT, INPUT);
pinMode(Pin_Mode_Button, INPUT);
pinMode(Pin_Backlight, OUTPUT); 

digitalWrite(Pin_Backlight, HIGH);

lcd.setCursor(0, 0);
lcd.print("**  2m 600W SSPA  **");
lcd.setCursor(0, 1);
lcd.print("* ON4KHG 2014 V2.0 *");

delay(3500);

digitalWrite(Pin_Backlight, LOW);
  
//Creation of the special LCD characters 
  
lcd.createChar(0, Col1);
lcd.createChar(1, Col2);
lcd.createChar(2, Col3);
lcd.createChar(3, Col4);
lcd.createChar(4, Col5);
lcd.createChar(5, Empty);
lcd.createChar(6, deg);

//Research of the OneWire device (temperature sensor)

if (!ds.search(addr)) {
      lcd.clear(); lcd.print("No more addrs");
      delay(1000);
      ds.reset_search();
      return;
  }
  if (OneWire::crc8(addr, 7) != addr[7]) {
      lcd.clear(); lcd.print("CRC not valid!");
      delay(1000);
      return;
  }
}    

//******************************************************************************
//******************************* LOOP *****************************************
//******************************************************************************

void loop() {

buttonState = digitalRead(Pin_Mode_Button);  
  if (buttonState != lastButtonState)  {       
     if (buttonState == HIGH) {buttonPushCounter++;} //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
        lcd.clear();}  
lastButtonState = buttonState;               
if (buttonPushCounter==3) {buttonPushCounter=1;}

buttonState = digitalRead(Pin_Mode_Button);
  if (buttonState == HIGH) {     //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXx
     digitalWrite(Pin_Backlight, HIGH);
        BacklightTime = millis ();} 
  else if (((millis() -  BacklightTime) > 60000)) {digitalWrite(Pin_Backlight, LOW);} 
 

switch (buttonPushCounter) {
  
  case 1:  //Display temperature
    
    getTemp();
    printTemp();
    delay(200);
    break;
  
  case 2:  //Dispaly bar graph  
       
    Calculation_Power(); 
    PTTState = digitalRead(Pin_PTT);
     if(PTTState == HIGH) {    //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
    
      val = analogRead(Analog_Input_Port);
      val=val+offset;
      percent = val/1024.0*100.0;
      lcd.setCursor(0,0);
      a=length/100*percent;
        if (a>=1) {
          for (i=1;i<a;i++) {
          lcd.print((char)4);
          b=i;}
          a=a-b;}
          peace=(a*5)+10;
    
    FS_Meter();
    
    for (i=0;i<(length-b);i++) {lcd.print((char)5);}
       lcd.setCursor(0, 1);    
       lcd.print("*  Power : ");     

     if(timer>interval)  {   
       timer-=interval;     
       lcd.setCursor(11, 1);  
       lcd.print(Power_W,3);
       lcd.setCursor(16, 1);
       lcd.print(" W *");}   
} 
     else {
       lcd.setCursor(0, 0);  
       lcd.print(" Status : Ready *** ");
       lcd.setCursor(0, 1);  
       lcd.print(" ******** Mode : RX ");}   
       delay(10);
       break;
} 
}

//******************************************************************************
//******************************* SUB-ROUTINES *********************************
//******************************************************************************

void Calculation_Power(){
  Value = analogRead(Analog_Input_Port);
  Voltage = (Value*5.00)/1024;
  Power_W = (pow(Voltage,2));
  Power_W = Power_W*12.209;
  Power_W = Power_W+(87.805*Voltage)+24.229;
}

void FS_Meter() {
   
switch (peace) {

  case 10:
    break;

  case 11:
    lcd.print((char)0);
    break;

  case 12:
    lcd.print((char)1);
    break;

  case 13:
    lcd.print((char)2);
    break;

  case 14:
    lcd.print((char)3);
    break;
}
}
  
void getTemp() {
  int foo, bar;
  ds.reset();
  ds.select(addr);
  ds.write(0x44,1);
  present = ds.reset();
  ds.select(addr);    
  ds.write(0xBE);

  for ( j = 0; j < 9; j++) {data[j] = ds.read();}
  
  LowByte = data[0];
  HighByte = data[1];
  TReading = (HighByte << 8) + LowByte;
  SignBit = TReading & 0x8000;  
  
  if (SignBit) {TReading = -TReading;}
  Tc_100 = (6 * TReading) + TReading / 4;    // multiply by (100 * 0.0625) or 6.25
  Whole = Tc_100 / 100;          // separate off the whole and fractional portions
  Fract = Tc_100 % 100;
  if (Fract > 49) {
    if (SignBit) {
      --Whole;} 
    else {
      ++Whole;}
}

    if (SignBit) {
      bar = -1;}
    else {
    bar = 1;}
}

void printTemp() {
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Heatsink temp.:");
  lcd.setCursor(0,1);
  lcd.print("********************");
  lcd.setCursor(16,0);   
  if (SignBit) {  
     lcd.print("-");}
  Whole=Whole;
  lcd.print(Whole);
  lcd.print((char)6);
  lcd.print("C");
}
 
  
